package cornell.ece4760.timesystem;

import java.util.ArrayList;

import android.app.AlertDialog;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

/**
 * Activity for Remote Control tab, which sets up the six buttons Power, 
 * Play/Pause, Prev, Next, Enter, and Upload. The activity is also in 
 * charge of parsing rows from the database into a 1KB file for upload.
 * Note: Partial code adapted from Google's Bluetooth Chat API example,
 * see project README.
 * 
 * @author Andy Liang
 */
public class Bluetooth extends Activity {
	// Debugging
	private static final String TAG = "Bluetooth";
	private static final boolean D = true;

	// Message types sent from the BluetoothService Handler
	public static final int MESSAGE_STATE_CHANGE = 1;
	public static final int MESSAGE_READ = 2;
	public static final int MESSAGE_WRITE = 3;
	public static final int MESSAGE_DEVICE_NAME = 4;
	public static final int MESSAGE_TOAST = 5;

	// Key names received from the BluetoothService Handler
	public static final String DEVICE_NAME = "device_name";
	public static final String TOAST = "toast";

	// Intent request codes
	private static final int REQUEST_CONNECT_DEVICE = 1;
	private static final int REQUEST_ENABLE_BT = 2;

	// Layout Views
	private Button mPowerButton;
	private Button mPrevButton;
	private Button mNextButton;
	private Button mEnterButton;
	private Button mPPButton;
	private Button mUploadButton;

	// Local Bluetooth adapter grabbed from TimeSystem class
	private BluetoothAdapter mBluetoothAdapter = null;
	// Member object for the chat services
	private BluetoothService mService = null;

	private final String dbName= "RoutineDB";
	private final String tableName= "RoutineContent";


	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		if(D) Log.e(TAG, "+++ ON CREATE +++");
		// set up view content from layout bluetooth.xml
		setContentView(R.layout.bluetooth);
		// set up buttons and get reference to bluetooth service and bluetooth adapter
		setupView();
	}

	// Method to set up view content and get bluetooth service and adapter reference
	private void setupView() {
		// set up OnClickListeners for the six buttons:
		// first five send the appropriate instruction bytes
		// upload button calls sendAlert()
		mPowerButton = (Button) findViewById(R.id.button_power);
		mPowerButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Send a message using content of the edit text widget
				sendMessage(113);
			}
		});
		mPrevButton = (Button) findViewById(R.id.button_prev);
		mPrevButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Send a message using content of the edit text widget
				sendMessage(112);
			}
		});
		mNextButton = (Button) findViewById(R.id.button_next);
		mNextButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Send a message using content of the edit text widget
				sendMessage(110);
			}
		});
		mEnterButton = (Button) findViewById(R.id.button_enter);
		mEnterButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Send a message using content of the edit text widget
				sendMessage(101);
			}
		});
		mPPButton = (Button) findViewById(R.id.button_pp);
		mPPButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Send a message using content of the edit text widget
				sendMessage(97);
			}
		});
		mUploadButton = (Button) findViewById(R.id.button_upload);
		mUploadButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Send a message using content of the edit text widget
				sendAlert();
			}
		});

		// get service adapter from TimeSystem instance
		if (null == mBluetoothAdapter)
			mBluetoothAdapter= TimeSystem.mBluetoothAdapter;


		// get service from TimeSystem instance
		if (null == mService)
			mService= TimeSystem.mService;
	}

	private void sendAlert() {

		// prepare the alert box
		AlertDialog.Builder alertbox = new AlertDialog.Builder(this);

		// set the message to display
		alertbox.setMessage("Upload?");

		// add a neutral button to the alert box and assign a click listener
		alertbox.setNeutralButton("Ok", new DialogInterface.OnClickListener() {

			// click listener on the alert box
			public void onClick(DialogInterface arg0, int arg1) {
				// the button was clicked
				// Toast.makeText(getApplicationContext(), "OK button clicked", Toast.LENGTH_LONG).show();
				sendMessage(117);
				try {
					System.out.println("sleeping");
					Thread.sleep(2000);
					System.out.println("done sleeping");
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				sendFile();
			}
		});

		// show it
		alertbox.show();

	}


	private void sendFile() {


		SQLiteDatabase database= this.openOrCreateDatabase(dbName, MODE_PRIVATE, null);
		ArrayList<Integer> tasks= new ArrayList<Integer>();
		int byteCounter= 0;

		for (int i=1; i<=32; i++) {
			Cursor c = database.rawQuery("SELECT * FROM " +
					tableName +
					" WHERE RoutineID = " + i + " ORDER BY Seq ASC", null);
			// if routine does not even have a header, insert header
			if (c.getCount() == 0) {
				ContentValues updateRoutine = new ContentValues();
				updateRoutine.put("RoutineID", i);
				updateRoutine.put("Seq", 0);
				updateRoutine.put("Byte1", 128);
				updateRoutine.put("Byte2", 0);
				updateRoutine.put("Label", "Routine Header");
				updateRoutine.put("Message", "");
				try {
					database.insertOrThrow(tableName, null, updateRoutine);	
				} catch (SQLException e) {
					String error= String.format("Error: %s", e.toString());
					Toast.makeText(this, error, Toast.LENGTH_SHORT).show();
					Log.e(TAG,error);
				}
				// re-query
				c = database.rawQuery("SELECT * FROM " +
						tableName +
						" WHERE RoutineID = " + i + " ORDER BY Seq ASC", null);
			} 

			if (c != null ) {
				if  (c.moveToFirst()) {
					do {
						int b1= c.getInt(c.getColumnIndex("Byte1"));
						int b2= c.getInt(c.getColumnIndex("Byte2"));
						int seq= c.getInt(c.getColumnIndex("Seq"));
						tasks.add(b1);
						byteCounter+=1;
						if ((b1 <= 128 && seq > 0) || (b1 == 137)) { // if counting or b1 is display remaining cycle
							tasks.add(b2);
							byteCounter+=1;
						}
					}while (c.moveToNext());
				} 
			}
			c.close();

			// write last PROMPT FOR ROUTINE byte for this routine
			tasks.add(131);
			byteCounter+=1;
		}

		database.close();

		// if did not exceed 1024 bytes, pad with zeros
		if (byteCounter <= 1024) {
			sendMessage(131); // initial prompt for routine
			for (int i= 0; i< 1024; i++) {
				if (i < tasks.size()) {
					sendMessage(tasks.get(i));
				} else if (i == tasks.size()) {
					sendMessage(224); // add a jump to routine 1 at the end
				} else {
					sendMessage(0);
				}
			}

		} else if (byteCounter > 1024) {
			Toast.makeText(this, "Error: Data exceeded 1KB" , Toast.LENGTH_SHORT).show();
		}

	}

	@Override
	public synchronized void onPause() {
		super.onPause();
		if(D) Log.e(TAG, "- ON PAUSE -");
	}

	@Override
	public void onStop() {
		super.onStop();
		if(D) Log.e(TAG, "-- ON STOP --");
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		/* // Stop the Bluetooth chat services
        if (mService != null) mService.stop();
        if(D) Log.e(TAG, "--- ON DESTROY ---");
		 */
	}

	private void ensureDiscoverable() {
		if(D) Log.d(TAG, "ensure discoverable");
		if (mBluetoothAdapter.getScanMode() !=
			BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
			Intent discoverableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
			discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
			startActivity(discoverableIntent);
		}
	}

	/**
	 * Sends a message.
	 * @param message  A string of text to send.
	 */
	private void sendMessage(int message) {
		// Check that we're actually connected before trying anything
		if (null == mService || mService.getState() != BluetoothService.STATE_CONNECTED) {
			Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT).show();
			//System.out.println(Integer.toBinaryString(message));
			byte send= (byte) (message & 0xff);
			System.out.println(((Byte)send).toString());
			return;
		}

		// Get the message bytes and tell the BluetoothService to write
		//byte[] send = message.getBytes();
		byte[] send = new byte[1];
		send[0] = (byte) (message & 0xff);
		mService.write(send);

	}


	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		if(D) Log.d(TAG, "onActivityResult " + resultCode);
		switch (requestCode) {
		case REQUEST_CONNECT_DEVICE:
			// When DeviceListActivity returns with a device to connect
			if (resultCode == Activity.RESULT_OK) {
				// Get the device MAC address
				String address = data.getExtras()
				.getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
				// Get the BLuetoothDevice object
				System.out.println(mBluetoothAdapter);
				BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
				// Attempt to connect to the device
				mService.connect(device);
			}
			break;
		case REQUEST_ENABLE_BT:
			// When the request to enable Bluetooth returns
			if (resultCode == Activity.RESULT_OK) {
				// Bluetooth is now enabled, so set up a chat session
				setupView();
			} else {
				// User did not enable Bluetooth or an error occured
				Log.d(TAG, "BT not enabled");
				Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
				//finish();
			}
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.option_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.scan:
			// Launch the DeviceListActivity to see devices and do scan
			Intent serverIntent = new Intent(this, DeviceListActivity.class);
			startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
			return true;
		case R.id.discoverable:
			// Ensure this device is discoverable by others
			ensureDiscoverable();
			return true;
		}
		return false;
	}

}